from fastapi import APIRouter, status
from typing import List, Optional
from sqlalchemy.orm import Session
from fastapi import Depends
from core.database import get_db
from models.book import Book
from schemas.book import UserBookCreate, UserBookUpdate
from helpers.book_helpers import get_user_books, create_user_book, get_user_book_by_id, update_user_book, delete_user_book
import uuid
from fastapi import HTTPException

router = APIRouter()

@router.post("/user-books", status_code=status.HTTP_201_CREATED, response_model=Book)
async def create_book(
    book_data: UserBookCreate,
    db: Session = Depends(get_db)
):
    return create_user_book(db=db, book_data=book_data)

@router.get("/user-books", status_code=200, response_model=List[Book])
async def get_books(
    title: Optional[str] = None,
    author: Optional[str] = None,
    genre: Optional[str] = None,
    db: Session = Depends(get_db)
):
    return get_user_books(db=db, title=title, author=author, genre=genre)

@router.get("/user-books/{book_id}", status_code=200, response_model=Book)
async def get_book(
    book_id: uuid.UUID,
    db: Session = Depends(get_db)
):
    book = get_user_book_by_id(db=db, book_id=book_id)
    if not book:
        raise HTTPException(status_code=404, detail="Book not found")
    return book

@router.put("/user-books/{book_id}", status_code=200, response_model=Book)
async def update_book(
    book_id: uuid.UUID,
    book_data: UserBookUpdate,
    db: Session = Depends(get_db)
):
    updated_book = update_user_book(db=db, book_id=book_id, book_data=book_data)
    if not updated_book:
        raise HTTPException(status_code=404, detail="Book not found")
    return updated_book

@router.delete("/user-books/{book_id}", status_code=204)
async def delete_book(
    book_id: uuid.UUID,
    db: Session = Depends(get_db)
):
    if not delete_user_book(db=db, book_id=book_id):
        raise HTTPException(status_code=404, detail="Book not found")
    return None